#!/usr/bin/ksh

#copyright	"%c%"
#ident	"@(#)opensrc:src/cups-1.1.19/chprnsys.sh /main/uw714mp/1"

#
# Change Print System
#
# chprnsys [cups|lp]
#
# Exit codes:
#	0 - successful
#	1 - script failed
#	2 - changed print system but unable to start it
#

LABEL="UX:$(basename $0)"

PKGDIR=/var/sadm/pkg			# System package directory
BACKUP=${PKGDIR}/cups/install/backup	# Backup script
RESTORE=${PKGDIR}/cups/install/restore	# Restore script

TMP=${TMPDIR:=/var/tmp}/chprnsys_$$	# Temp files prefix.
FILES=${TMP}_files			# List of files overwritten by CUPS.
DELFILES=${TMP}_delfiles		# List of files deleted by CUPS.
NEWFILES=${TMP}_newfiles		# List of new files by CUPS.
INSTALLF=${TMP}_installf		# List of pathnames for installf.
REMOVEF=${TMP}_removef			# List of pathnames for removef.

PATH=/sbin:/usr/sbin:/usr/bin:${PATH}	# Just in case.

usage()
{
	pfmt -l ${LABEL} -s action "Usage: $(basename $0) [cups|lp]\n"
	exit 1
}

clean()
{
	[ -z "${DEBUG}" ] &&
		rm -f ${TMP}_*
}

_trap()
{
	trap '' 1 2 9 15
	pfmt -l ${LABEL} -s warn "Interrupted. Print system may be unusable.\n"
	clean
	exit 1
}

# main

# Determine active print system.
if [ -f /etc/dinit.d/S80cups ]
then
	ACTIVE="cups"
else
	ACTIVE="lp"
fi

# Chprnsys without any argument displays active print system.
[ $# -eq 0 ] && {
	pfmt -l ${LABEL} -s info \
		"%s is the active print system.\n" ${ACTIVE} 2>&1
	exit 0
}

case "$1" in
cups|lp)
	;;
*)	pfmt -l ${LABEL} -s error "Invalid print system -- %s\n" $1
	usage ;;
esac

# Check backup script.
[ -f ${BACKUP} ] || {
	pfmt -l ${LABEL} -s error "Unable to find backup script.\n"
	exit 1
}

# Check restore script.
[ -f ${RESTORE} ] || {
	pfmt -l ${LABEL} -s error "Unable to find restore script.\n"
	exit 1
}

trap '_trap' 1 2 9 15

CUPS=/usr/lib/cups

#
# List of files to be changed.
# + LP print commands
# + BSD compatible print commands
# + SCOadmin printer/printjob interfaces
#
cat > ${FILES} <<-EOF
s base /usr/bin/cancel=${CUPS}/bin/cancel
s base /usr/bin/disable=${CUPS}/bin/disable
s base /usr/bin/enable=${CUPS}/bin/enable
s base /usr/bin/lp=${CUPS}/bin/lp
s base /usr/sbin/lpadmin=${CUPS}/sbin/lpadmin
s lp /usr/bin/lpstat=${CUPS}/bin/lpstat
s lp /usr/lib/accept=${CUPS}/sbin/accept
s lp /usr/lib/lpadmin=${CUPS}/sbin/lpadmin
s lp /usr/lib/reject=${CUPS}/sbin/reject
s lp /usr/sbin/accept=${CUPS}/sbin/accept
s lp /usr/sbin/lpmove=${CUPS}/sbin/lpmove
s lp /usr/sbin/reject=${CUPS}/sbin/reject
s bsdcompat /usr/ucb/lpc=${CUPS}/sbin/lpc
s bsdcompat /usr/ucb/lpq=${CUPS}/bin/lpq
s bsdcompat /usr/ucb/lpr=${CUPS}/bin/lpr
s bsdcompat /usr/ucb/lprm=${CUPS}/bin/lprm
s scoadmin /usr/lib/scoadmin/printer/printerGUI=${CUPS}/scoadmin/printerGUI
s scoadmin /usr/lib/scoadmin/printjob/printjobGUI=${CUPS}/scoadmin/printjobGUI
EOF

#
# List of files to be deleted.
# + LP startup/shutdown scripts
# + LP print commands man pages
# + BSD compatible print commands man pages
#
cat > ${DELFILES} <<-EOF
f base /etc/init.d/lp
l base /etc/dinit.d/S80lp
l base /etc/rc0.d/K20lp
l base /etc/rc1.d/K20lp
f baseman /usr/man/html.1/lp.1.html
f baseman /usr/man/html.1/lpstat.1.html
f baseman /usr/man/html.1M/accept.1M.html
f baseman /usr/man/html.1M/enable.1M.html
f baseman /usr/man/html.1M/lp.cat.1M.html
f baseman /usr/man/html.1M/lpadmin.1M.html
f baseman /usr/man/html.1M/lpfilter.1M.html
f baseman /usr/man/html.1M/lpforms.1M.html
f baseman /usr/man/html.1M/lpmove.1M.html
f baseman /usr/man/html.1M/lpsched.1M.html
f baseman /usr/man/html.1M/lpsystem.1M.html
f baseman /usr/man/html.1M/lpusers.1M.html
f baseman /usr/man/html.1Mbsd/lpc.1Mbsd.html
f baseman /usr/man/html.1bsd/lpq.1bsd.html
f baseman /usr/man/html.1bsd/lpr.1bsd.html
f baseman /usr/man/html.1bsd/lprm.1bsd.html
f baseman /usr/man/html.1bsd/lptest.1bsd.html
EOF

#
# List of new files. (links)
# + CUPS startup script (we do not need K-links in /etc/rc[01].d/ directories)
# + CUPS specific user and admin print commands.
# + CUPS (en and fr) man pages.
#
cat > ${NEWFILES} <<-EOF
l cups /etc/dinit.d/S80cups=/etc/init.d/cups
s cups /usr/bin/cups-config=${CUPS}/bin/cups-config
s cups /usr/bin/cupstestppd=${CUPS}/bin/cupstestppd
s cups /usr/bin/lpoptions=${CUPS}/bin/lpoptions
s cups /usr/bin/lppasswd=${CUPS}/bin/lppasswd
s cups /usr/sbin/cupsaddsmb=${CUPS}/sbin/cupsaddsmb
s cups /usr/sbin/cupsd=${CUPS}/sbin/cupsd
s cups /usr/sbin/lpinfo=${CUPS}/sbin/lpinfo
s cups /usr/man/man.1/backend.1=${CUPS}/man/man.1/backend.1
s cups /usr/man/man.1/cupstestppd.1=${CUPS}/man/man.1/cupstestppd.1
s cups /usr/man/man.1/filter.1=${CUPS}/man/man.1/filter.1
s cups /usr/man/man.1/lp.1=${CUPS}/man/man.1/lp.1
s cups /usr/man/man.1/lpoptions.1=${CUPS}/man/man.1/lpoptions.1
s cups /usr/man/man.1/lppasswd.1=${CUPS}/man/man.1/lppasswd.1
s cups /usr/man/man.1/lpq.1=${CUPS}/man/man.1/lpq.1
s cups /usr/man/man.1/lpr.1=${CUPS}/man/man.1/lpr.1
s cups /usr/man/man.1/lprm.1=${CUPS}/man/man.1/lprm.1
s cups /usr/man/man.1/lpstat.1=${CUPS}/man/man.1/lpstat.1
s cups /usr/man/man.3/cups-config.3=${CUPS}/man/man.3/cups-config.3
s cups /usr/man/man.5/classes.conf.5=${CUPS}/man/man.5/classes.conf.5
s cups /usr/man/man.5/cupsd.conf.5=${CUPS}/man/man.5/cupsd.conf.5
s cups /usr/man/man.5/mime.convs.5=${CUPS}/man/man.5/mime.convs.5
s cups /usr/man/man.5/mime.types.5=${CUPS}/man/man.5/mime.types.5
s cups /usr/man/man.5/printers.conf.5=${CUPS}/man/man.5/printers.conf.5
s cups /usr/man/man.8/accept.8=${CUPS}/man/man.8/accept.8
s cups /usr/man/man.8/cups-lpd.8=${CUPS}/man/man.8/cups-lpd.8
s cups /usr/man/man.8/cups-polld.8=${CUPS}/man/man.8/cups-polld.8
s cups /usr/man/man.8/cupsaddsmb.8=${CUPS}/man/man.8/cupsaddsmb.8
s cups /usr/man/man.8/cupsd.8=${CUPS}/man/man.8/cupsd.8
s cups /usr/man/man.8/enable.8=${CUPS}/man/man.8/enable.8
s cups /usr/man/man.8/lpadmin.8=${CUPS}/man/man.8/lpadmin.8
s cups /usr/man/man.8/lpc.8=${CUPS}/man/man.8/lpc.8
s cups /usr/man/man.8/lpinfo.8=${CUPS}/man/man.8/lpinfo.8
s cups /usr/man/man.8/lpmove.8=${CUPS}/man/man.8/lpmove.8
s cups /usr/man/man.1/cancel.1=${CUPS}/man/man.1/cancel.1
s cups /usr/man/man.8/disable.8=${CUPS}/man/man.8/disable.8
s cups /usr/man/man.8/reject.8=${CUPS}/man/man.8/reject.8
s cups /usr/man/fr/man.1/backend.1=${CUPS}/man/fr/man.1/backend.1
s cups /usr/man/fr/man.1/cupstestppd.1=${CUPS}/man/fr/man.1/cupstestppd.1
s cups /usr/man/fr/man.1/filter.1=${CUPS}/man/fr/man.1/filter.1
s cups /usr/man/fr/man.1/lp.1=${CUPS}/man/fr/man.1/lp.1
s cups /usr/man/fr/man.1/lpoptions.1=${CUPS}/man/fr/man.1/lpoptions.1
s cups /usr/man/fr/man.1/lppasswd.1=${CUPS}/man/fr/man.1/lppasswd.1
s cups /usr/man/fr/man.1/lpq.1=${CUPS}/man/fr/man.1/lpq.1
s cups /usr/man/fr/man.1/lpr.1=${CUPS}/man/fr/man.1/lpr.1
s cups /usr/man/fr/man.1/lprm.1=${CUPS}/man/fr/man.1/lprm.1
s cups /usr/man/fr/man.1/lpstat.1=${CUPS}/man/fr/man.1/lpstat.1
s cups /usr/man/fr/man.3/cups-config.3=${CUPS}/man/fr/man.3/cups-config.3
s cups /usr/man/fr/man.5/classes.conf.5=${CUPS}/man/fr/man.5/classes.conf.5
s cups /usr/man/fr/man.5/cupsd.conf.5=${CUPS}/man/fr/man.5/cupsd.conf.5
s cups /usr/man/fr/man.5/mime.convs.5=${CUPS}/man/fr/man.5/mime.convs.5
s cups /usr/man/fr/man.5/mime.types.5=${CUPS}/man/fr/man.5/mime.types.5
s cups /usr/man/fr/man.5/printers.conf.5=${CUPS}/man/fr/man.5/printers.conf.5
s cups /usr/man/fr/man.8/accept.8=${CUPS}/man/fr/man.8/accept.8
s cups /usr/man/fr/man.8/cups-lpd.8=${CUPS}/man/fr/man.8/cups-lpd.8
s cups /usr/man/fr/man.8/cups-polld.8=${CUPS}/man/fr/man.8/cups-polld.8
s cups /usr/man/fr/man.8/cupsaddsmb.8=${CUPS}/man/fr/man.8/cupsaddsmb.8
s cups /usr/man/fr/man.8/cupsd.8=${CUPS}/man/fr/man.8/cupsd.8
s cups /usr/man/fr/man.8/enable.8=${CUPS}/man/fr/man.8/enable.8
s cups /usr/man/fr/man.8/lpadmin.8=${CUPS}/man/fr/man.8/lpadmin.8
s cups /usr/man/fr/man.8/lpc.8=${CUPS}/man/fr/man.8/lpc.8
s cups /usr/man/fr/man.8/lpinfo.8=${CUPS}/man/fr/man.8/lpinfo.8
s cups /usr/man/fr/man.8/lpmove.8=${CUPS}/man/fr/man.8/lpmove.8
s cups /usr/man/fr/man.1/cancel.1=${CUPS}/man/fr/man.1/cancel.1
s cups /usr/man/fr/man.8/disable.8=${CUPS}/man/fr/man.8/disable.8
s cups /usr/man/fr/man.8/reject.8=${CUPS}/man/fr/man.8/reject.8
EOF

case $1 in
cups)
	#
	# If cups is already the active print system, the code below will
	# verify that all pieces all in placem and fix those that are not.
	#
	
	# Stop print service.
	echo "## Stopping ${ACTIVE} print service."
	/etc/init.d/${ACTIVE} stop > /dev/null

	# Backup files.
	echo "## Backing up lp files."
	cut -f3 -d' ' ${FILES} ${DELFILES} |
	cut -f1 -d= |
	sh ${BACKUP}
	[ $? -ne 0 ] && {
		pfmt -l ${LABEL} -s error "Unable to create backup.\n"
		clean
		exit 1
	}

	# Delete lp files.
	echo "## Deleting lp files." 
	while read type pkg pathname
	do
		# Skip files belonging to uninstalled packages.
		pkginfo -q ${pkg} || continue

		# Delete file.
		rm -f ${pathname}

		# Record pathname entry for removef.
		echo ${pathname} >> ${REMOVEF}_${pkg}

		# Keep track of packages for removef.
		echo ${pkg}

	done < ${DELFILES} |
	sort -u > ${REMOVEF}_package_names

	# Install cups files.
	echo "## Installing cups files."
	cat ${FILES} ${NEWFILES} |
	while read type pkg pathname
	do
		target=$(echo ${pathname} | cut -f1 -d=)
		source=$(echo ${pathname} | cut -f2 -d=)

		# Link target to source.
		if [ ${type} = "l" ]
		then
			ln -f ${source} ${target}
		else
			rm -f ${target}
			ln -s ${source} ${target}
		fi

		# Record pathname entry for installf.
		echo "${target}=${source} ${type}" >> ${INSTALLF}_${pkg}

		# Keep track of packages for installf.
		echo ${pkg}

	done |
	sort -u > ${INSTALLF}_package_names

	# Remove privileges from /usr/usb/lpr command.
	[ -f /usr/ucb/lpr ] && /sbin/filepriv -d /usr/ucb/lpr

	# Update system contents file.
	echo "## Updating system information."

	#
	# Commented out code to update system contents file for deleted files.
	#
	# If two or more packages refer to these files then removef -f does not
	# delete the entry.  Also, since the file has already been deleted, it
	# is unable to properly update the entry for the other packages.
	#
	# Besides, we use removef with the name of the first package to install
	# these files, and during restore we use installf with the name of the
	# last package to install these files.  This is obviously an error.
	#
	# These problems are not trival to fix and for now we can live with
	# pkgchk errors on these deleted files.
	#
	# while read pkg
	# do
	# 	cat ${REMOVEF}_${pkg} |
	# 	removef ${pkg} - > /dev/null
	# 	removef -f ${pkg}
	# done < ${REMOVEF}_package_names

	while read pkg
	do
		cat ${INSTALLF}_${pkg} |
		installf -c ${pkg} cups - 2>> /var/sadm/install/logs/cups.out
		installf -f -c ${pkg} cups
	done < ${INSTALLF}_package_names

	# Configure man pages.
	echo "## Configuring man pages."
	makewhatis \
		/usr/man/man.1 /usr/man/man.3 \
		/usr/man/man.5 /usr/man/man.8 \
		/usr/man/fr/man.1 /usr/man/fr/man.3 \
		/usr/man/fr/man.5 /usr/man/fr/man.8 >/dev/null 2>&1

	# Configure documentation.
	echo "## Configuring documentation."
	(
	PR_group=HelpTopics/PRhome/PRgroup
	pkginfo -q cupsdoc && {
		doctool --add --metaonly $PR_group/PR_cups.desktop
		doctool --add --metaonly $PR_group/PR_cups.de.desktop
		doctool --add --metaonly $PR_group/PR_cups.fr.desktop
	}
	doctool --remove --metaonly $PR_group/PRConfiguring_Hewlett-Packard.desktop
	doctool --remove --metaonly $PR_group/PR_admin.desktop
	doctool --remove --metaonly $PR_group/PR_advanced.desktop
	doctool --remove $PR_group/PR_sched.desktop
	) >/dev/null 2>&1

	# Remove temp files.
	clean

	# Start CUPS server.
	/etc/init.d/cups start > /dev/null 2>&1

	if /etc/init.d/cups status | grep -q "scheduler is running"
	then
		rc=0
	else
		rc=2
	fi
	;;

lp)
	# If LP is the active print server then nothing to do.
	[ ${ACTIVE} = "lp" ] && {
		pfmt -l ${LABEL} -s info "%s is the active print system.\n" lp
		exit 0
	}

	# Stop CUPS server.
	echo "## Stopping cups print service."
	/etc/init.d/cups stop > /dev/null 2>&1

	echo "## Restoring saved lp files."
	sh ${RESTORE}
	case $? in
	0)	;;
	1)	pfmt -l ${LABEL} -s error \
			"Backup not found -- cannot switch back to lp.\n"
		exit 1 ;;
	*)	pfmt -l ${LABEL} -s error \
			"Restore failed -- cannot switch back to lp.\n"
		exit 1 ;;
	esac

	# Update system contents file.
	echo "## Updating system information."
	cut -f3 -d' ' ${FILES} ${NEWFILES} |
	cut -f1 -d= |
	removef cups - |
	while read file
	do
		# echo "${file} <pathname removed>"
		rm -f ${file}
	done
	removef -f cups

	# Configure man pages.
	echo "## Configuring man pages."

	# Remove cached copies of man pages.
	grep '/usr/man/' ${NEWFILES} |
	cut -f3 -d' ' |
	cut -f1 -d= |
	sed -e 's/man\./cat./' |
	xargs rm -f

	makewhatis >/dev/null 2>&1

	# Configure documentation.
	echo "## Configuring documentation."
	(
	PR_group=HelpTopics/PRhome/PRgroup
	pkginfo -q cupsdoc && {
		doctool --remove --metaonly $PR_group/PR_cups.desktop
		doctool --remove --metaonly $PR_group/PR_cups.de.desktop
		doctool --remove --metaonly $PR_group/PR_cups.fr.desktop
	}
	doctool --add --metaonly $PR_group/PRConfiguring_Hewlett-Packard.desktop
	doctool --add --metaonly $PR_group/PR_admin.desktop
	doctool --add --metaonly $PR_group/PR_advanced.desktop
	doctool --add $PR_group/PR_sched.desktop
	) >/dev/null 2>&1

	# Start LP print server.
	if /etc/init.d/lp start | grep -q "Print services started"
	then
		rc=0
	else
		rc=2
	fi
	;;
esac

case $rc in
0)	pfmt -l ${LABEL} -s info "Started %s print service.\n" $1
	;;
2)	pfmt -l ${LABEL} -s error "Unable to start %s print service.\n" $1
	;;
esac

exit $rc
